unit uxFrame;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs;

type

  TxFrame = class(TFrame)
  end;

  TxFrameClass = class of TxFrame;

  TxFrameInfo = class
  private
    fFrame: TxFrame;
    procedure ShowFrame(pWinControl: TWinControl);
  public
    constructor Create(pFrameClass: TxFrameClass);
    destructor Destroy; override;
    property Frame: TxFrame read fFrame;
  end;

  TxFrameManager = class
  private
    fFrameList: TList;
    function GetFrameCount: Integer;
    function GetFrameInfo(pIndex: Integer): TxFrameInfo;
    procedure HideFrames;
  public
    constructor Create;
    destructor Destroy; override;
    procedure RegisterFrame(pFrameClass: TxFrameClass);
    procedure ShowFrame(pIndex: Integer; pWinControl: TWinControl);
    property FrameCount: Integer read GetFrameCount;
    property FrameInfos[pIndex: Integer]: TxFrameInfo read GetFrameInfo;
  end;

implementation

{$R *.dfm}

{ TxFrameInfo }

constructor TxFrameInfo.Create(pFrameClass: TxFrameClass);
begin
  inherited Create;
  fFrame := pFrameClass.Create(nil);
end;

destructor TxFrameInfo.Destroy;
begin
  // pri unistavanju informacije o frameu treba unistiti i sam frame
  fFrame.Free;
  inherited;
end;

procedure TxFrameInfo.ShowFrame(pWinControl: TWinControl);
begin
  // prikazivanje framea na prosledjenoj kontroli
  fFrame.Parent := pWinControl;
  fFrame.Align := alTop;
end;

{ TxFrameManager }

constructor TxFrameManager.Create;
begin
  fFrameList := TList.Create;
end;

destructor TxFrameManager.Destroy;
  var
    i: Integer;
begin
  // unistavanje svih informacije o frameovima
  for i := 0 to FrameCount - 1 do
    FrameInfos[i].Free;
  // unistavanje same liste
  fFrameList.Free;
  inherited;
end;

function TxFrameManager.GetFrameInfo(pIndex: Integer): TxFrameInfo;
begin
  Result := TxFrameInfo(fFrameList[pIndex]);
end;

function TxFrameManager.GetFrameCount: Integer;
begin
  Result := fFrameList.Count;
end;

procedure TxFrameManager.HideFrames;
  var
    i: Integer;
begin
  // sakrivanje svih frameove
  for i := 0 to FrameCount - 1 do
    FrameInfos[i].Frame.Parent := nil;
end;

procedure TxFrameManager.RegisterFrame(pFrameClass: TxFrameClass);
  var
    lFrameInfo: TxFrameInfo;
begin
  // kreiranje informacije o frameu i unos u listu
  lFrameInfo := TxFrameInfo.Create(pFrameClass);
  fFrameList.Add(lFrameInfo);
end;

procedure TxFrameManager.ShowFrame(pIndex: Integer; pWinControl: TWinControl);
  var
    lFrameInfo: TxFrameInfo;
    i, lHeight: Integer;
    lCanDraw: Boolean;
begin
  // sakrivanje svih frameova
  HideFrames;
  // postavljanje pocetnih vrednosti
  lHeight := pWinControl.Height;
  lCanDraw := True;
  i := pIndex;
  // prikazivanje frameova dok ih ima i dok ima mesta za njihovo prikazivanje
  while (i < GetFrameCount) and lCanDraw do begin
    // pronadji frame po indeksu
    lFrameInfo := FrameInfos[i];
    if lFrameInfo <> nil then begin
      // frame je pronadjen
      // da li ima mesta da se prikaze?
      lCanDraw := lHeight >= lFrameInfo.Frame.Height;
      // ako ima mesta za prikaz framea onda se i prikazuje
      if lCanDraw then
        lFrameInfo.ShowFrame(pWinControl);
      // smanjivanje prostora za prikaz framea
      lHeight := lHeight - lFrameInfo.Frame.Height;
    end;
    // prelazak na sledeci frame
    Inc(i);
  end;
end;

end.

